# -*- coding: utf-8 -*-
#
# Copyright 2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Test Ubunto SSO Wizard."""

from PyQt4 import QtGui
from twisted.internet import defer

from ubuntu_sso.qt import ubuntu_sso_wizard
from ubuntu_sso.qt.tests import (
    BaseTestCase,
    FakedObject,
)


class ReturnCodeTestCase(BaseTestCase):
    """Test the return codes."""

    @defer.inlineCallbacks
    def setUp(self):
        yield super(ReturnCodeTestCase, self).setUp()
        self.patch(ubuntu_sso_wizard.sys, 'exit', self._set_called)
        self.app_name = 'app_name'
        self.ui = ubuntu_sso_wizard.UbuntuSSOWizard(self.app_name)

    def test_closing_main_window(self):
        """When closing the main window, USER_CANCELLATION is returned."""
        self.ui.close()
        self.assertEqual(self._called,
            ((ubuntu_sso_wizard.USER_CANCELLATION,), {}))

    # pylint: disable=W0212
    def test_close_wizard_reaching_success_page(self):
        """Closing the window in success page, USER_SUCCESS is returned."""
        email = 'email@example.com'
        self.ui._move_to_success_page(self.app_name, email)
        self.ui.close()
        self.assertEqual(self._called,
            ((ubuntu_sso_wizard.USER_SUCCESS,), {}))
    # pylint: enable=W0212


class UbuntuSSOWizardTestCase(BaseTestCase):

    """Test Case for UbuntuSSOWizard"""

    @defer.inlineCallbacks
    def setUp(self):
        yield super(UbuntuSSOWizardTestCase, self).setUp()
        self.app_name = 'app_name'
        self.ui = ubuntu_sso_wizard.UbuntuSSOWizard(self.app_name)

    # pylint: disable=W0212
    def test_move_to_success_page_state(self):
        """Test _move_to_success_page method."""
        self.patch(self.ui, 'setButtonLayout', self._set_called)
        self.ui._move_to_success_page(self.app_name)
        self.assertEqual(self.ui._next_id, -1)
        self.assertEqual(self.ui._next_id, -1)
        self.assertEqual(self._called,
            (([QtGui.QWizard.Stretch, QtGui.QWizard.FinishButton],), {}))
        self.assertTrue(self.ui.button(QtGui.QWizard.FinishButton).isEnabled())
        self.assertEqual(self.ui._done, ubuntu_sso_wizard.USER_SUCCESS)
    # pylint: enable=W0212

    # pylint: disable=W0212, E1101
    def test_sing_in_canceled_connection(self):
        """Test if sign in canceled signal calls close and not reject."""
        exposed_methods = [
            'close',
            'reject']
        self.patch(FakedObject, "exposed_methods", exposed_methods)
        faked_object = FakedObject()
        self.patch(self.ui, "closeEvent", faked_object.close)
        self.patch(self.ui, "reject", faked_object.reject)
        self.ui.show()
        self.addCleanup(self.ui.hide)
        self.ui.sign_in_page.ui.cancel_button.clicked.emit(True)
        self.assertIn('close', faked_object._called)
        self.assertNotIn('reject', faked_object._called)
    # pylint: enable=W0212, E1101
